@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Conformal Systems LLC <info@conformal.com>
+ * Copyright (c) 2011, 2012 Conformal Systems LLC <info@conformal.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -17,13 +17,19 @@
 #ifndef CT_SOCKET_H
 #define CT_SOCKET_H
 
-#include <event.h>
+#include <ct_threads.h>
+
+#include <event2/event.h>
+
+#ifndef evutil_socket_t
+#define evutil_socket_t int
+#endif
 
 /* provide a typedef for the libevent callback type */
-typedef void (eventcb_ty)(int, short, void *);
+typedef void (eventcb_ty)(evutil_socket_t, short, void *);
 
 typedef void (msgdeliver_ty)(void *, struct ct_header *, void *);
-typedef void (msgcomplete_ty)(void *, struct ct_header *, void *, int);
+typedef int (msgcomplete_ty)(void *, struct ct_header *, void *, int);
 typedef void (limitio_ty)(void *, size_t);
 
 struct ct_assl_io_ctx;
@@ -64,8 +70,8 @@ struct ct_io_ctx {
 	ct_ioctx_alloc_func		*io_ioctx_alloc;
 	ct_ioctx_free_func		*io_ioctx_free;
 	TAILQ_HEAD(, ct_io_queue)	io_o_q;
-	struct event			io_ev_rd;
-	struct event			io_ev_wr;
+	struct event			*io_ev_rd;
+	struct event			*io_ev_wr;
 	int				io_i_fd;
 	int				io_o_fd;
 	int				io_i_state;
@@ -79,6 +85,8 @@ struct ct_io_ctx {
 	int				io_user_flow_control;
 	int				io_write_io_enabled;
 
+	CT_LOCK_STORE(io_lock);
+
 	/* stats */
 	uint64_t			io_write_bytes;
 	uint64_t			io_write_count;
@@ -114,6 +122,8 @@ struct ct_assl_io_ctx {
 
 	int				io_write_io_enabled;
 
+	CT_LOCK_STORE(io_lock);
+
 	/* stats */
 	uint64_t			io_write_bytes;
 	uint64_t			io_write_count;
@@ -145,7 +155,8 @@ int			ct_assl_writev_op(struct ct_assl_io_ctx *,
 			    struct ct_header *, struct ct_iovec *, int);
 int			ct_io_writev_op(struct ct_io_ctx *, struct ct_header *,
 			    struct ct_iovec *, int);
-int			ct_io_connect_fd_pair(struct ct_io_ctx *ctx, int, int);
+int			ct_io_connect_fd_pair(struct ct_io_ctx *ctx, int, int,
+			    struct event_base *);
 void			ct_io_disconnect(struct ct_io_ctx *);
 void			ct_assl_disconnect(struct ct_assl_io_ctx *);
 
