@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Conformal Systems LLC <info@conformal.com>
+ * Copyright (c) 2011, 2012 Conformal Systems LLC <info@conformal.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -49,6 +49,7 @@ struct ct_settings {
 	double			*cs_fval;
 	long long		*cs_szval;
 	struct ct_special	*cs_s;
+	int			cs_secure;
 };
 
 struct ct_header {
@@ -165,6 +166,8 @@ struct ct_read {
  */
 #define C_HDR_O_WRITE		(16)
 #define C_HDR_O_WRITE_REPLY	(17)
+#define C_HDR_O_WRITE_EXS_ENOSPACE	(0)
+extern char	*c_hdr_write_reply_ex_errstr[];
 struct ct_write_reply {
 	u_char			cwr_digest[SHA_DIGEST_LENGTH];
 } __packed;
@@ -180,11 +183,19 @@ struct ct_metadata_footer {
 /* stuff */
 int	ct_get_password(char *, size_t, char *, int);
 int	ct_savecore(void);
+size_t	ct_str_repeat(char *dest, size_t buf_size, const char *src, int repeat);
 
 /* compression */
-int		ct_init_compression(uint16_t);
-int		ct_uncompress(uint8_t *, uint8_t *, size_t, size_t *);
-int		ct_compress(uint8_t *, uint8_t *, size_t, size_t *);
+struct ct_compress_ctx;
+struct ct_compress_ctx
+		*ct_init_compression(uint16_t);
+void		ct_cleanup_compression(struct ct_compress_ctx *);
+int		ct_uncompress(struct ct_compress_ctx *, uint8_t *, uint8_t *,
+		    size_t, size_t *);
+int		ct_compress(struct ct_compress_ctx *, uint8_t *, uint8_t *,
+		    size_t, size_t *);
+uint16_t	ct_compress_type(struct ct_compress_ctx *);
+size_t		ct_compress_bounds(struct ct_compress_ctx *, size_t);
 
 /* digest */
 void		ct_sha1(uint8_t *, uint8_t *, size_t);
@@ -198,6 +209,11 @@ void		ct_sha1_setup(SHA_CTX *);
 void		ct_sha1_add(uint8_t *, SHA_CTX *, size_t);
 void		ct_sha1_final(uint8_t *, SHA_CTX *);
 
+#define SHA512_DIGEST_STRING_LENGTH ((SHA512_DIGEST_LENGTH *2) + 1)
+void		ct_sha512_setup(SHA512_CTX *);
+void		ct_sha512_add(uint8_t *, SHA512_CTX *, size_t);
+void		ct_sha512_final(uint8_t *, SHA512_CTX *);
+
 /* error handling */
 char		*ct_header_strerror(struct ct_header *);
 
@@ -226,9 +242,9 @@ void		ct_dump_block(uint8_t *p, size_t sz);
 int		ct_base64_encode(int, uint8_t *, size_t, uint8_t *, size_t);
 
 /* Metadata */
-#define CT_MAX_MD_FILENAME	(256)	/* 255 + terminating NUL */
-#define	CT_MD_TAG_MAXLEN	(129)	/* 128 + terminating NUL */
-#define	CT_MD_TAG_REJECTCHRS	"?!*/\\\'\""
+#define	CT_MAX_MD_FILENAME	(256)	/* 255 + terminating NUL */
+#define	CT_CTFILE_MAXLEN	CT_MAX_MD_FILENAME
+#define	CT_CTFILE_REJECTCHRS	"?!*/\\\'\""
 
 /* filename handling */
 char	*ct_remove_ext(char *path);
@@ -239,6 +255,9 @@ int     ct_make_full_path(char *, mode_t);
 /* pipe handling */
 int	 ct_set_pipe_nonblock(int);
 
+#define CT_PATHSEP	'/'
+#define CT_PATHSEP_STR	"/"
+
 /* cli parsing */
 struct ct_cli_cmd {
 	char			*cc_cmd;	/* command name */
@@ -256,6 +275,11 @@ __dead void		ct_cli_usage(struct ct_cli_cmd *, struct ct_cli_cmd *);
 struct ct_cli_cmd	*ct_cli_validate(struct ct_cli_cmd *, int *, char ***);
 void			ct_cli_execute(struct ct_cli_cmd *, int *, char ***);
 
+/* cert bundle */
+#define CT_CERT_BUNDLE_LOGIN_FAILED	(-1000)
+int			ct_get_cert_bundle(const char *, const char *, char **,
+			    size_t *);
+
 /* Debug log levels */
 #define CTUTIL_LOG_SOCKET	(0x1)
 #define CTUTIL_LOG_CONFIG	(0x2)
